/***************************************************************************
                          highscores.h  -  description
                             -------------------
    begin                : Tue Apr 5 2005
    copyright            : (C) 2005 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef HIGHSCORES_H
#define HIGHSCORES_H

#include "tpbase.h"

#include <qstring.h>
#include <qstringlist.h>
#include <qmap.h>
#include <qptrlist.h>
#include <qdatetime.h>

/**
This class keeps track of the highscores

@author Thomas Friedrichsmeier
*/
class Highscores : public TPBase {
public:
	Highscores ();

	~Highscores ();

	struct Highscore {
		int start_level;
		int end_level;
		float score;
		QString name;
		QDate date;
	};

/** shows a modal dialog with all highscores */
	void showHighscores (const QString &mission);
/** this function adds a new score to the highscore table. You don't need to check whether the score really is a high score. This function will take care
of checking that, do nothing if the score does not make it to the highscores, pop up a dialog, if it does. Also you don't need to specify a date / time */
	void addScore (const QString &mission_file, int start_level, int end_level, float score);
private:
friend class HighscoresDialog;
/** typedef for a list of highscores for a single mission */
	typedef QPtrList<Highscore> ScoreList;
/** where the highscores (for all missions) are kept in memory */
	QMap <QString, ScoreList*> scores_map;

/** saves the current highscore table for the given mission. Internal - addScore includes a call to saveHighscores if needed. */
	void saveHighscores (const QString &mission);
/** initialization: read highscores (for all missions) */
	void readHighscores ();
/** read highscores for a single mission */
	ScoreList *readHighscores (const QString &mission);
/** free up the data */
	void clearHighscores ();
};

#include <kdialogbase.h>
class QComboBox;
class QListView;

/**
This is the dialog used to display the highscore table(s)

@author Thomas Friedrichsmeier
*/
class HighscoresDialog : public KDialogBase {
	Q_OBJECT;
public:
	HighscoresDialog (Highscores *parent, const QString &initial_mission);
	~HighscoresDialog ();
public slots:
	void missionSelected (int index);
private:
	QComboBox *combo;
	QListView *listview;
	QStringList mission_files;
	Highscores *hs;
};

#endif
